function [u_k,p1_k,p2_k,v_k]=ITBV_restoration_rd2(I,u_k,F,gamma,lambda,alpha,kmax,PSF,m1,m2,pm,pn)
%The algorithm similar as the one in the paper "An Infeasible Primal-Dual Algorithm for Total bounded 
%Variation-Based Inf-Convolution-Type Image Restoration", BUT FOR L1-NORM DATA-FITTING TERM.
%For fixed alpha.
%With noise detector. For salt-and-pepper noise, using adaptive median filter.
%Only input the kernel of the blurring operator
%Written by Yiqiu Dong on Feb. 2, 2009



%% Initialization, (u_k, p_k)
p1_k=0.000001*ones(m1,m2);
p2_k=p1_k;
v_k=ones(m1,m2);
k=1;

aPSF=PSF(end:-1:1,end:-1:1);

K_u_k=convb(u_k,PSF,pm,pn);
nablax_u_k=[diff(u_k,1,2),zeros(m1,1)];
nablay_u_k=[diff(u_k,1,1);zeros(1,m2)];
nu_Xi_t=function_xi_t(nablax_u_k,nablay_u_k);
m_gamma=max(gamma,nu_Xi_t);
diff_u_k=K_u_k-I;
abs_diff_u_k=abs(diff_u_k);
m_lambda=max(lambda,abs_diff_u_k);
res1=diff_u_k-m_lambda.*v_k;     %Residual of the first equation
res2=[p1_k(:,1),diff(p1_k,1,2)]+[p2_k(1,:);diff(p2_k,1,1)]-convb(F.*v_k,aPSF,pm,pn);   %Residual of the second equation
res3=[m_gamma.*p1_k;m_gamma.*p2_k]-alpha*[nablax_u_k;nablay_u_k];    %Residual of the third equation
resk=norm(res3,'fro')+norm(res2,'fro')+norm(res1,'fro');    %Residual of the equations


%% Update (u_k, p_k)
while  resk(k)>=resk(1)*1e-3  && k<=kmax 
        
    %Indicative function X_A_(k+1)
    X_A_gamma=active_set_A(nu_Xi_t, gamma);
    X_A_lambda=active_set_A(abs_diff_u_k,lambda);
    
    %Enforce condition |p|_s<=alpha and |divp|<=1
    p_Xi_t=function_xi_t(p1_k,p2_k);
    temp=alpha./max(alpha,p_Xi_t);
    p1_k=p1_k.*temp;
    p2_k=p2_k.*temp;
    v_k=v_k./max(1,abs(v_k));
    
    %Computation of A_k
    A_k=F.*(1-v_k.*X_A_lambda.*sign(diff_u_k))./m_lambda;  % the vector for the diagonal matrix
    
    %Computation of B_k
    [Ba_k,Bb_k,Bc_k,Bd_k]=condition_C(nablax_u_k,nablay_u_k,p1_k,p2_k,alpha,X_A_gamma,nu_Xi_t);
        
    %Computation of f_k
    temp1=diff_u_k./m_lambda;
    temp21=nablax_u_k./m_gamma;
    temp22=nablay_u_k./m_gamma;
    f_k=-convb(F.*temp1,aPSF,pm,pn)+alpha*[temp21(:,1),diff(temp21,1,2)]+alpha*[temp22(1,:);diff(temp22,1,1)];

    %System Solution
    delta_u=bicgstabm4(PSF,aPSF,A_k,Ba_k./m_gamma,Bb_k./m_gamma,Bc_k./m_gamma,Bd_k./m_gamma,zeros(m1,m2),f_k,30,resk(k)/resk(1),m1,m2,pm,pn);

        
    %Update u_k and p_k
    u_k=u_k+delta_u;
    v_k=temp1+A_k.*convb(delta_u,PSF,pm,pn);
    nablax_delta_u=[diff(delta_u,1,2),zeros(m1,1)];
    nablay_delta_u=[diff(delta_u,1,1);zeros(1,m2)];
    p1_k=(alpha*nablax_u_k+Ba_k.*nablax_delta_u+Bb_k.*nablay_delta_u)./m_gamma;
    p2_k=(alpha*nablay_u_k+Bc_k.*nablax_delta_u+Bd_k.*nablay_delta_u)./m_gamma;
       
    %Calculate nabla_u_k a matrix of 2mx1
    K_u_k=convb(u_k,PSF,pm,pn);
    nablax_u_k=[diff(u_k,1,2),zeros(m1,1)];
    nablay_u_k=[diff(u_k,1,1);zeros(1,m2)];
    nu_Xi_t=function_xi_t(nablax_u_k,nablay_u_k);
    m_gamma=max(gamma,nu_Xi_t);
    diff_u_k=K_u_k-I;
    abs_diff_u_k=abs(diff_u_k);
    m_lambda=max(lambda,abs_diff_u_k);
    
    %Residual
    res1=diff_u_k-m_lambda.*v_k;     %Residual of the first equation
    res2=[p1_k(:,1),diff(p1_k,1,2)]+[p2_k(1,:);diff(p2_k,1,1)]-convb(F.*v_k,aPSF,pm,pn);   %Residual of the second equation
    res3=[m_gamma.*p1_k;m_gamma.*p2_k]-alpha*[nablax_u_k;nablay_u_k];    %Residual of the third equation
    resk(k+1)=norm(res3,'fro')+norm(res2,'fro')+norm(res1,'fro');    %Residual of the equations
    %diff=norm(u_k1-u_k)/norm(u_k1);
       
    k=k+1;
    
end
%t=toc
k
